/*
 * The routines in this file provide support for ANSI style terminals
 * over a serial line. The serial I/O services are provided by routines in
 * "termio.c". It compiles into nothing if not an ANSI device.
 */

#define	termdef	1			/* don't define "term" external */

#include        <stdio.h>
#include	"estruct.h"
#include        "edef.h"

#if     ANSI

#if	AMIGA
#define NROW    23                      /* Screen size.                 */
#define NCOL    77                      /* Edit if you want to.         */
#else
#define NROW    25                      /* Screen size.                 */
#define NCOL    80                      /* Edit if you want to.         */
#endif
#define	MARGIN	8			/* size of minimim margin and	*/
#define	SCRSIZ	64			/* scroll size for extended lines */
#define BEL     0x07                    /* BEL character.               */
#if EBCDIC
#define ESC     0x27                    /* ESC character.               */
#else
#define ESC     0x1B                    /* ESC character.               */
#endif

extern  int     ttopen(void);           /* Forward references.          */
extern  int     ttgetc(void);
extern  int     ttputc(int c);
extern  int     ttflush(void);
extern  int     ttclose(void);
extern  int     ansimove(int row, int col);
extern  int     ansieeol(void);
extern  int     ansieeop(void);
extern  int     ansibeep(void);
extern  int     ansiopen(void);
extern	int	ansirev(int state);
extern	int	ansigetc(void);
extern  int     ansiparm(register int n);
/*
 * Standard terminal interface dispatch table. Most of the fields point into
 * "termio" code.
 */
TERM    term    = {
        NROW-1,
        NCOL,
	MARGIN,
	SCRSIZ,
        ansiopen,
        ttclose,
        ansigetc,
        ttputc,
        ttflush,
        ansimove,
        ansieeol,
        ansieeop,
        ansibeep,
	ansirev
};

int ansimove(int row, int col)
{
        ttputc(ESC);
        ttputc('[');
        ansiparm(row+1);
        ttputc(';');
        ansiparm(col+1);
        ttputc('H');
}

int ansieeol(void)
{
        ttputc(ESC);
        ttputc('[');
        ttputc('K');
}

int ansieeop(void)
{
        ttputc(ESC);
        ttputc('[');
        ttputc('J');
}

int ansirev(int state)		/* change reverse video state */

	/* state: TRUE = reverse, FALSE = normal */

{
	ttputc(ESC);
	ttputc('[');
	ttputc(state ? '7': '0');
	ttputc('m');
}

int ansibeep(void)
{
        ttputc(BEL);
        ttflush();
}

int ansiparm(register int n)
{
        register int    q;

        q = n/10;
        if (q != 0)
                ansiparm(q);
        ttputc((n%10) + '0');
}

#endif

int ansiopen(void)
{
#if     V7
        register char *cp;
        char *getenv();

        if ((cp = getenv("TERM")) == NULL) {
                puts("Shell variable TERM not defined!");
                exit(1);
        }
        if (strcmp(cp, "vt100") != 0) {
                puts("Terminal type not 'vt100'!");
                exit(1);
        }
#endif
	revexist = TRUE;
        ttopen();
}

int ansigetc(void)
{
        int c;

        c = ttgetc();
        if (c == ESC) {
            c = ttgetc();
            if (c == ESC) {
                return (ESC);
            }
            if (c == '[') {
                c = ttgetc();
                if ((c == 'A')
                    || (c == 'B')
                    || (c == 'C')
                    || (c == 'D')
                    || (c == 'F')
                    || (c == 'H')
                   ) {

                    c |= SPEC;
                }
                else if ((c == '3')
                         || (c == '5')
                         || (c == '6'))
                {
                    ttgetc(); /* assume we got '~' */
                    c |= SPEC;
#if 0
                    if (c == '~')
                    {
                        c = SPEC | '3';
                    }
#endif
                }
                else if (c == '1')
                {
                    ttgetc(); /* assume we got ';' */
                    ttgetc(); /* assume we got '5' */
                    c = ttgetc();
                    if (c == 'F')
                    {
                        c = SPEC | '>';
                    }
                    else if (c == 'H')
                    {
                        c = SPEC | '<';
                    }
                }
            }
        }
        return (c);
}
