/*********************************************************************/
/*                                                                   */
/*  This Program Written By Paul Edwards.                            */
/*  Released to the public domain.                                   */
/*                                                                   */
/*********************************************************************/
/*********************************************************************/
/*                                                                   */
/*  brkpds - break a pds into its constituents.                      */
/*                                                                   */
/*  This program takes the following parameters:                     */
/*  1) Name of dataset (PDS with member specified)                   */
/*  2) Address of pointer to PDS name to be stored.                  */
/*     (Initialize to NULL on first call).                           */
/*  3) Address of pointer which will point to member name or pattern */
/*     (Initialize to NULL on first call).                           */
/*                                                                   */
/*  This program returns 1 if successful, otherwise zero.            */
/*                                                                   */
/*  e.g. brkpds("lib.public(lines)",&pds,&mem);                      */
/*                                                                   */
/*********************************************************************/

#include <string.h>
#include <stddef.h>
#include <stdlib.h>

int brkpds(char *dsn, char **pdsp, char **memp)
{
    int len;
    ptrdiff_t memdif, dsndif, startmem, endmem, startdsn, enddsn;
    char *endmemp, *startmemp;

    len = strlen(dsn);
    endmemp = strrchr(dsn,')');

    if (endmemp == NULL) endmem = 0;
    else endmem = endmemp - dsn;

    startmemp = strchr(dsn,'(');
    if (startmemp == NULL) startmem = 0;
    else startmem = startmemp - dsn;

    if ((endmem == 0) && (startmem == 0)) memdif = 1;
    else
    {
        memdif = endmem-startmem-1;
        if ((memdif < 1) || (memdif > 8)) return (0);
    }

    *memp = realloc(*memp,memdif+1);
    if ((endmem == 0) && (startmem == 0))
        /* strcpy(*memp,"*") */ return (0);
    else
    {
        strncpy(*memp,dsn+startmem+1,memdif);
        *(*memp+memdif) = '\0';
    }

    if (startmem != 0) enddsn = startmem;
    else if (startmem > 0) enddsn = startmem - 1;
    else enddsn = len;

    if (dsn[0] == '\'') startdsn = 1;
    else startdsn = 0;

    dsndif = enddsn-startdsn;
    if (dsndif > 44) return (0);
    *pdsp = realloc(*pdsp,dsndif+1+2*startdsn);
    strncpy(*pdsp,dsn,dsndif+startdsn);
    if (startdsn == 1) *(*pdsp+dsndif+1) = '\'';
    *(*pdsp+dsndif+2*startdsn) = '\0';
    return (1);
}
