/*********************************************************************/
/*                                                                   */
/*  This Program Written by Paul Edwards.                            */
/*  Released to the public domain                                    */
/*                                                                   */
/*********************************************************************/
/*********************************************************************/
/*                                                                   */
/*  disjcl - generate the disassembly JCL                            */
/*                                                                   */
/*********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAXBUF 300

static char buf[MAXBUF];
static char mod[MAXBUF];
static char mac[MAXBUF];
static char src[MAXBUF];
static char csect[MAXBUF];
static char procbuf[MAXBUF];

static FILE *proc;
static FILE *control;
static FILE *jclf;
static char *jcl;

static char fnm[FILENAME_MAX];

int main(int argc, char **argv)
{
    int c;

    if (argc < 4)
    {
        printf("usage: disjcl <control> <jcl> <proc>\n");
        printf("reads control file generating JCL members\n");
        return (EXIT_FAILURE);
    }
    
    control = fopen(*(argv + 1), "r");
    if (control == NULL)
    {
        printf("failed to open %s\n", *(argv + 1));
        return (EXIT_FAILURE);
    }
    
    jcl = *(argv + 2);
    
    proc = fopen(*(argv + 3), "r");
    if (proc == NULL)
    {
        printf("failed to open %s\n", *(argv + 3));
        return (EXIT_FAILURE);
    }
    
    strcpy(mod, "");
    while (fgets(buf, sizeof buf, control) != NULL)
    {
        if (memcmp(buf, "MOD", 3) == 0)
        {
            sscanf(buf + 3, "%s", mod);
            /* put in default maclib */
            strcpy(mac, "V500000");
        }
        else if (memcmp(buf, "%MOD", 4) == 0)
        {
            /* this is a module that is up-to-date */
            /* so everything needs to be ignored until the next MOD */
            strcpy(mod, "");
        }
        else if (strcmp(mod, "") == 0)
        {
            /* scan until a MOD is found */
            continue;
        }
        else if (memcmp(buf, "MAC", 3) == 0)
        {
            sscanf(buf + 3, "%s", mac);
        }
        else if (memcmp(buf, "SRC", 3) == 0)
        {
            strcpy(csect, "");
            sscanf(buf + 3, "%s %s", src, csect);
            if (strcmp(csect, "") == 0)
            {
                strcpy(csect, src);
            }
            
            sprintf(fnm, "%s(%s)", jcl, src);
            jclf = fopen(fnm, "w");
            if (jclf == NULL)
            {
                printf("failed to open %s\n", fnm);
                return (EXIT_FAILURE);
            }
            fprintf(jclf, "//MVSBLD   JOB CLASS=C,REGION=0K\n"
                    "//*\n");
            rewind(proc);
            while (fgets(procbuf, sizeof procbuf, proc) != NULL)
            {
                fputs(procbuf, jclf);
            }
            fprintf(jclf,
                    "//%-8s EXEC MVSBLD,SRC=%s,\n"
                    "// MACLIB='MVSSRC.BACKMAC.%s'\n"
                    "//*\n"
                    "//DISASM1.DISIN DD *\n"
                    "MODULE   %s\n"
                    "CSECT    %s\n"
                    "OPTIONS  ROUND\n",
                    src, src, mac, src, csect);
            while (1)
            {
                c = getc(control);
                if (c == '-')
                {
                    fgets(buf, sizeof buf, control);
                    fprintf(jclf, "%s", buf);
                }
                else
                {
                    ungetc(c, control);
                    break;
                }
            }
            fprintf(jclf,
                    "/*\n"
                    "// DD DSN=&&FILLER,DISP=(OLD,PASS)\n"
                    "//*\n"
                    "//DISASM2.DISIN DD *\n"
                    "MODULE   %s\n"
                    "CSECT    %s\n"
                    "OPTIONS  ROUND\n"
                    "/*\n"
                    "// DD DSN=&&FILLER,DISP=(OLD,PASS)\n"
                    "//*\n"
                    "//\n"
                    , mod, csect);
            fclose(jclf);
        }
    }
    return (0);
}
